import unittest

from canonical_args import structure



class TestObj(object):
	pass

class TestCase_CheckArgs(unittest.TestCase):

	def test_check_args(self):
		spec = {
			"args": [
				{
					"name": "arg1",
					"type": int,
					"values": None
				}
			],
			"kwargs": {
				"kwarg1": {
					"type": float,
					"values": None
				}
			}
		}

		structure.checkspec(spec, [1], {"kwarg1": 33.1})

	def test_check_args_fail(self):
		spec = {
			"args": [
				{
					"name": "arg1",
					"type": int,
					"values": None
				}
			],
			"kwargs": {
				"kwarg1": {
					"type": float,
					"values": None
				}
			}
		}

		try:
			structure.checkspec(spec, [1], {"kwarg1": "string"})  # bad type
		except AssertionError:
			pass
		else:
			self.fail("should have thrown AssertionError")

	def test_check_args_with_eval(self):
		spec = {
			"args": [
				{
					"name": "arg1",
					"type": "int",
					"values": None
				}
			],
			"kwargs": {
				"kwarg1": {
					"type": "one([int, NoneType])",
					"values": {
						"int": ">0",
						"NoneType": None
					},
					"required": True
				}
			}
		}

		structure.checkspec(spec, [1], {"kwarg1": None})


	def test_check_complex_args(self):
		from canonical_args.check import one, cls, NoneType, TypeType
		argspec = {
			"args": [
				{
					"name": "arg1",
					"type": "one([int, float, str])",
					"values": {
						"int": ">0&&!=5",
						"float": "<5.3||>7.8",
						"str": ["A", "B", "C", "X"]
					}
				},
				{
					"name": "arg2",
					"type": dict,
					"values": {
						"subkey1": {
							"type": cls('test_structure.TestObj'),
							"values": None,
							"required": True
						},
						"subkey2": {
							"type": TypeType,
							"values": [int, float],
							"required": False
						}
					}
				}
			],
			"kwargs": {
				"kwarg1": {
					"type": one([int, float, dict, NoneType]),
					"values": {
						"int": "!=100",
						"float": "range(0, 99)",
						"dict": {
							"subkey1": {
								"type": int,
								"values": ">=0&&!=10",
								"required": True
							},
							"subkey2": {
								"type": float,
								"values": None,
								"required": True
							}
						},
						"NoneType": None
					}
				},
				"kwarg2": {
					"type": one([int, NoneType]),
					"values": {
						"int": "(>10||<10)&&!=5",
						"NoneType": None
					}
				}
			}
		}

		structure.checkspec(
			argspec,
			[0.113, {"subkey1": TestObj(), "subkey2": int}],
			{
				"kwarg1": {
					"subkey1": 5,
					"subkey2": 459.132
				},
				"kwarg2": None
			})

class TestCase_NestedOnes(unittest.TestCase):

	def test_nested_choice_of_one(self):
		argspec = {
			"args": [
				{
					"name": 'arg1',
					"type": "one([int, dict])",
					"values": {
						"int": None,
						"dict": {
							"subkey1": {
								"type": "one([str, int])",
								"values": {
									"str": None,
									"int": None
								}
							}
						}
					}
				}
			]
		}

		structure.checkspec(
			argspec,
			[{"subkey1": 1}])
		structure.checkspec(argspec, [1])
		structure.checkspec(argspec, [{"subkey1": "xyz"}])

	def test_nested_list_choice_of_one(self):
		argspec = {
			"args": [
				{
					"name": "arg1",
					"type": "one([int, float, list([int, int])])",
					"values": {
						"int": None,
						"float": None,
						"list": [None, None]
					}
				}
			]
		}

		structure.checkspec(argspec, [[1, 1]])

class TestCase_BadLength(unittest.TestCase):

	def test_args_too_short(self):
		argspec = {
			"args": [
				{
					"name": "arg1",
					"type": str,
					"values": None
				},
				{
					"name": "arg2",
					"type": str,
					"values": None
				}
			]
		}

		try:
			structure.checkspec(argspec, ["A"])
		except TypeError:
			pass
		else:
			self.fail("should have thrown TypeError, too short")

	def test_args_too_many(self):
		argspec = {
			"args": [
				{
					"name": "arg1",
					"type": str,
					"values": None
				}
			]
		}

		try:
			structure.checkspec(argspec, ["A", "B"])
		except TypeError:
			pass
		else:
			self.fail("should have thrown TypeError, too many")

	def test_kwargs_extra_arg(self):
		argspec = {
			"args": [],
			"kwargs": {
				"kwarg1": {
					"type": int,
					"values": None
				}
			}
		}

		try:
			structure.checkspec(argspec, [], {"badkwarg": 15})
		except TypeError:
			pass
		else:
			self.fail("should have thrown TypeError, unrecognized kwarg")

	def test_missing_dict_subkey(self):
		argspec = {
			"args": [
				{
					"name": "arg1",
					"type": dict,
					"values": {
						"subkey1": {
							"type": int,
							"values": None,
							"required": True  # explicitly required
						},
						"subkey2": {
							"type": int,
							"values": None    # implicitly required
						},
						"subkey3": {
							"type": int,
							"values": None,
							"required": False  # not required
						}
					}
				}
			],
			"kwargs": {}
		}

		try:
			structure.checkspec(argspec, [{"subkey1": 15}])  #missing subkey2
		except AssertionError:
			pass
		else:
			self.fail("should have thrown AssertionError, missing subkey2")

	def test_missing_optional_subkey(self):
		argspec = {
			"args": [
				{
					"name": "arg1",
					"type": dict,
					"values": {
						"subkey1": {
							"type": int,
							"values": None,
							"required": True  # explicitly required
						},
						"subkey2": {
							"type": int,
							"values": None    # implicitly required
						},
						"subkey3": {
							"type": int,
							"values": None,
							"required": False  # not required
						}
					}
				}
			],
			"kwargs": {}
		}

		structure.checkspec(argspec, [{"subkey1": 15, "subkey2": 15}])


"""
Provides utilities for checking the structure of a list
or dict against provided structure mandates and details.
"""
from __future__ import absolute_import

import json

from . import check



def checkspec(spec, args=[], kwargs={}):

    def recurse(level, name, types, values, arg):

        subtype = check.eval_subtype(types)

        if isinstance(subtype, check.ChoiceOfOne):
            # argtype = type(arg)
            # subvalue = values[check.type_to_string(argtype)]

            index = None
            for i, st in enumerate(subtype):
                if isinstance(st, (list, dict)):
                    st = type(st)
                if isinstance(arg, st):
                    index = i
                    break

            argtype = subtype[index]
            subvalue = values[check.type_to_string(type(arg))]

            recurse(level+1,
                    name,
                    argtype,
                    subvalue,
                    arg)

        # structlist
        elif isinstance(subtype, list) and isinstance(values, list):
            arg = check.check_subtype(name, type(subtype), arg)

            # check for matching length
            if not len(arg) == len(subtype):
                raise AssertionError(
                    "arg `{}` expected of length {}. got arg"\
                    " of length {}".format(name, len(arg), len(subtype)))

            for index, val in enumerate(arg):
                recurse(level+1,
                        name+"#"+str(index),
                        subtype[index],
                        values[index],
                        val)

        # structdict
        elif subtype == dict and isinstance(values, dict):
            arg = check.check_subtype(name, subtype, arg)

            # check for required keys
            required = [key for key in values.keys() \
                        if "required" not in values[key] or \
                        values[key]["required"]]

            missing = set(required) - set(arg.keys())
            if len(missing) > 0:
                raise AssertionError(
                    "arg `{}` missing required keys {}".format(
                        name, missing))

            for key, val in arg.items():
                recurse(level+1,
                        name+"."+key,
                        values[key]["type"],
                        values[key]["values"],
                        val)

        # unstruct list
        elif subtype == list and values is None:
            arg = check.check_subtype(name, subtype, arg)

        # unstruct dict
        elif subtype == dict and values is None:
            arg = check.check_subtype(name, subtype, arg)

        # selector
        elif isinstance(values, list):
            arg = check.check_subtype(name, subtype, arg)
            check.check_value(name, subtype, values, arg)

        # native
        else:
            arg = check.check_subtype(name, subtype, arg)
            check.check_value(name, subtype, values, arg)


    # check for matching length
    if not len(args) == len(spec["args"]):
        raise TypeError(
            "expected {} positional arguments, got {}".format(
                len(spec["args"]), len(args)))

    for index, subspec in enumerate(spec["args"]):
        arg = args[index]
        recurse(0,
                subspec["name"],
                subspec["type"],
                subspec["values"],
                arg)


    for kw, kwarg in kwargs.items():
        if not kw in spec["kwargs"]:
            raise TypeError(
                "keyword arguments contain unknown key {}".format(
                    kw))

        subspec = spec["kwargs"][kw]
        recurse(0,
                kw,
                subspec["type"],
                subspec["values"],
                kwarg)



"""
Make top-level imports available.
"""
from __future__ import absolute_import

from .structure import checkspec
from .function import arg_spec, register_spec

# pkgutil namespace style packaging
from pkgutil import extend_path
__path__ = extend_path(__path__, __name__)



check_args = checkspec

__all__ = ["check_args",
		   "checkspec",
		   "arg_spec",
		   "register_spec"]



import unittest

from canonical_args import check



class TestObj(object):
	pass

class TestCase_EvalSubtype(unittest.TestCase):

	def test_type_native(self):
		self.assertEqual(check.eval_subtype("str"), str)

	def test_type_list(self):
		self.assertEqual(check.eval_subtype("list([int, str, float])"),
						 [int, str, float])

	def test_type_dict(self):
		self.assertEqual(check.eval_subtype("dict({'thing1': int})"),
											{"thing1": int})

	def test_type_object(self):
		self.assertEqual(check.eval_subtype("test_check.TestObj"), TestObj)

	def test_type_choice(self):
		self.assertEqual(check.eval_subtype("one([int, float])"),
						 check.ChoiceOfOne([int, float]))

	def test_choice_of_one_with_object(self):
		self.assertEqual(
			check.eval_subtype("one([int, cls('test_check.TestObj')])"),
			check.ChoiceOfOne([int, TestObj]))

	def test_type_none(self):
		self.assertEqual(check.eval_subtype("NoneType"), type(None))

	def test_complex_type_with_none(self):
		self.assertEqual(check.eval_subtype("[int, str, NoneType]"),
						 [int, str, type(None)])


class TestCase_CheckSubtype(unittest.TestCase):

	def test_type_native_no_cast(self):
		self.assertEqual(check.check_subtype("testArg",
											 int,
											 10),
						 10)

	def test_type_native_no_cast_fail(self):
		try:
			check.check_subtype("testArg", int, "string")
		except AssertionError:
			# successful failure.... oxymoron
			pass
		else:
			self.fail("should have thrown AssertionError")

	def test_type_native_cast(self):
		self.assertEqual(check.check_subtype("testArg",
											 int,
											 "10",
											 should_cast=True),
						 10)

	def test_type_native_cast_fail(self):
		try:
			check.check_subtype("testArg", int, "string", should_cast=True)
		except ValueError:
			# successful failure
			pass
		else:
			self.fail("should have thrown ValueError")

	def test_type_choice_of_one(self):
		self.assertEqual(check.check_subtype("testArg",
											 check.ChoiceOfOne([int, float]),
											 58.1),
						 58.1)

	def test_type_choice_of_one_object(self):
		obj = TestObj()
		self.assertEqual(check.check_subtype("testArg",
											 check.ChoiceOfOne([int,
											 					TestObj]),
											 obj),
						 obj)

	def test_type_choice_of_one_fail(self):
		try:
			check.check_subtype("testArg",
								check.ChoiceOfOne([int, float]),
								"string")
		except AssertionError:
			pass
		else:
			self.fail("should have thrown AssertionError")

	def test_type_choice_of_one_with_none(self):
		check.check_subtype("testArg",
							check.ChoiceOfOne([int, float, type(None)]),
							None)


class TestCase_CheckValueWhitelist(unittest.TestCase):

	def test_value_whitelist(self):
		check.check_value_whitelist("testArg",
									["A", "B", "C"],
									"B")

	def test_value_whitelist_fail(self):
		try:
			check.check_value_whitelist("testArg",
										["A", "B", "C"],
										"X")
		except AssertionError:
			pass
		else:
			self.fail("should have thrown AssertionError")

class TestCase_CheckValueRange(unittest.TestCase):

	def test_value_range_int(self):
		check.check_value_range("testArg",
								"range(0, 50)",
								25)

	def test_value_range_int_fail(self):
		try:
			check.check_value_range("testArg",
									"range(0, 50)",
									51)
		except AssertionError:
			pass
		else:
			self.fail("should have thrown AssertionError")

	def test_value_range_float(self):
		check.check_value_range("testArg",
								"range(0, 50.5)",
								25.1)

	def test_value_range_float_fail(self):
		try:
			check.check_value_range("testArg",
									"range(0, 50.5)",
									50.6)
		except AssertionError:
			pass
		else:
			self.fail("should have thrown AssertionError")

	def test_value_range_upper_limit(self):
		check.check_value_range("testArg",
								"range(0, 10)",
								10)

class TestCase_CheckValueComparison(unittest.TestCase):

	def test_value_comparison_gt(self):
		check.check_value_comparison("testArg",
									 ">5",
									 10)

	def test_value_comparison_gt_fail(self):
		try:
			check.check_value_comparison("testArg",
										 ">5",
										 5)
		except AssertionError:
			pass
		else:
			self.fail("should have thrown AssertionError")

	def test_value_comparison_lt(self):
		check.check_value_comparison("testArg",
							 "<5",
							 0)

	def test_value_comparison_lt_fail(self):
		try:
			check.check_value_comparison("testArg",
										 "<5",
										 5)
		except AssertionError:
			pass
		else:
			self.fail("should have thrown AssertionError")

	def test_value_comparison_gte(self):
		check.check_value_comparison("testArg",
									 ">=5",
									 5)

	def test_value_comparison_gte_fail(self):
		try:
			check.check_value_comparison("testArg",
										 ">=5",
										 4)
		except AssertionError:
			pass
		else:
			self.fail("should have thrown AssertionError")

	def test_value_comparison_lte(self):
		check.check_value_comparison("testArg",
									 "<=5",
									 5)

	def test_value_comparison_lte_fail(self):
		try:
			check.check_value_comparison("testArg",
										 "<=5",
										 6)
		except AssertionError:
			pass
		else:
			self.fail("should have thrown AssertionError")

	def test_value_comparison_formatted(self):
		check.check_value_comparison(
			"testArg",
			"(>0||<0)&&!=10",
			-10)

	def test_value_comparison_formatted_fail(self):
		try:
			check.check_value_comparison(
				"testArg",
				"(>0||<0)&&!=10",
				10)
		except AssertionError:
			pass
		else:
			self.fail("should have thrown AssertionError")


class TestCase_CheckValue(unittest.TestCase):

	def test_check_value_choice_of_one(self):
		subname = "testArg"
		subtypes = check.ChoiceOfOne([int, float])
		subvalues = {
			"int": ">0",
			"float": "<0"
		}
		subarg = 1

		check.check_value(subname, subtypes, subvalues, subarg)

	def test_check_value_choice_of_one_fail(self):
		subname = "testArg"
		subtypes = check.ChoiceOfOne([int, float])
		subvalues = {
			"int": ">0",
			"float": "<0"
		}
		subarg = 50.3  # is float, but >0

		try:
			check.check_value(subname, subtypes, subvalues, subarg)
		except AssertionError:
			pass
		else:
			self.fail("should have thrown AssertionError")

	def test_check_value_choice_of_one_with_object(self):
		subname = "testArg"
		subtypes = check.ChoiceOfOne([int, TestObj])
		subvalues = {
			"int": ">0",
			"test_check.TestObj": None
		}
		subarg = TestObj()

		check.check_value(subname, subtypes, subvalues, subarg)

	def test_check_value_choice_of_one_with_none(self):
		subname = "testArg"
		subtypes = check.ChoiceOfOne([int, type(None)])
		subvalues = {
			"int": ">0",
			"NoneType": None
		}
		subarg = None

		check.check_value(subname, subtypes, subvalues, subarg)





"""

"""
from __future__ import absolute_import

import types




def dynamic_import(class_string):
    """
    perform a python import, attempting first the new-style
    (aka ``getattr(module, "submodule")``), and falling back
    to old style (aka ``exec "from {} import {}".format(...)``)

    :param str class_string: a python import string pointing to
        the class. eg. ``package.module.submodule.AClass``
    :returns: ``types.ClassType``, the imported class.

    .. note :: this method can also work for packages and modules,
        not only classes.
    """
    package_name = class_string.split(".")[0]
    module_import = ".".join(p for p in class_string.split(".")[:-1])
    cla = class_string.split(".")[-1]

    try:
        mod = __import__(package_name)
        for comp in class_string.split(".")[1:]:
            mod = getattr(mod, comp)
        return mod
    except AttributeError, e:
        # cannot import using new fashion, try old
        exec "from {} import {}".format(module_import, cla)
        return eval(class_string)

def type_to_string(subtype):
    """
    convert a subtype (aka, ``types.*`` to a string)

        >>> type_to_string(type(10))
        'int'
        >>> type_to_string(package.module.SomeObject())
        'package.module.SomeObject'

    """
    subtype = str(subtype).replace("<type '", "").replace("'>", "")
    subtype = subtype.replace("<class '", "").replace("'>", "")
    return subtype

"""
special types used during eval
"""
class ChoiceOfOne(list):
    def __repr__(self):
        """
        evaluate to simply ``"one"`` for value lookup
        """
        x = super(ChoiceOfOne, self).__repr__()
        x = "<type 'one'>"
        return x

class StructuredListType(type):
    def __instancecheck__(self, instance):
        return type(instance) in [self, list]

class StructuredList(list):
    __metaclass__ = StructuredListType

    def __repr__(self):
        """
        evaluate to ``"structlist"`` for value lookup
        """
        x = super(StructuredList, self).__repr__()
        x = "<type 'structlist'>"
        return x

# eval type "one"
def one(subtype):
    assert isinstance(subtype, list)
    return ChoiceOfOne(eval_subtype(subtype))

# the structured list subtype
def structlist(subtype):
    assert isinstance(subtype, list)
    return StructuredList(eval_subtype(subtype))

# eval type "cls"
def cls(subtype):
    assert isinstance(subtype, str) or isinstance(subtype, unicode)
    # subtype = re.findall("\((.*)\)", subtype)[0]
    return dynamic_import(subtype)

# eval type "NoneType"
NoneType = types.NoneType

# eval type "Type"
TypeType = types.TypeType


def eval_subtype(subtype):
    """
    get the subtype from a subtype string.

    :param subtype: the string version of the type
    :type subtype: ``str`` or ``unicode``

    Acceptable values:

    ================  =====================================================
    Format            Use
    ================  =====================================================
    "int"             native types: str, float, int, bool, dict, list, etc.
    "one([])"         choice of one, can nest 'cls' calls here
    "structlist([])"  a structured list ``structlist([int, str, float])``
    "cls('')"         a class import string
    "NoneType"        evaluates to ``type(None)``
    "TypeType"        evaluates to ``<type 'type'>``
    ================  =====================================================
    """
    if isinstance(subtype, str) or isinstance(subtype, unicode):

        try:
            subtype = eval(subtype)
        except NameError:
            # assume this is a class for backwards compat
            subtype = dynamic_import(subtype)
        except SyntaxError:
            raise SyntaxError("unable to evaluate type string '{}'".format(
                subtype))

    elif isinstance(subtype, list):
        for i in range(0, len(subtype)):
            subtype[i] = eval_subtype(subtype[i])

    return subtype

def check_subtype(subname, subtype, subarg, should_cast=False):
    """
    check the type of ``subarg`` against the potential subtypes.

    :params str subname: the name of the arg (for error formatting)
    :params subtype: the type to check against
    :type subtype: ``<type 'type'>`` or ``canonical_args.check.ChoiceOfOne``
    :param subarg: the argument whose type is being checked
    :param bool should_cast: default False, when True, will attempt to
        cast the ``subarg`` to type ``subtype``. can only work for native
        types and ``cls`` with appropriate constructors.
    :returns: ``subarg`` or correct type (possibly altered if ``should_cast``)

    If ``subtype`` is of type ``ChoiceOfOne``, type of ``subarg`` will be
    compared against the list of potentials. Else, will be a straight
    ``isinstance`` check.
    """
    # choice of one
    if isinstance(subtype, ChoiceOfOne):
        try:
            assert type(subarg) in subtype
        except AssertionError:
            err = "expected argument '{}' of types {}. got {}.".format(
                subname, subtype, type(subarg))

            if None in subtype:
                # common error is to specify a type as "None",
                # but it MUST be "NoneType"
                err += "\nFound `None` in subtypes, did you mean `NoneType`?"

            raise AssertionError(err)

    # straight up type check
    else:
        try:
            assert isinstance(subarg, subtype)
        except AssertionError as e:
            if should_cast:
                try:
                    subarg = subtype(subarg)
                except ValueError as e:
                    raise ValueError("unable to cast argument '{}' of"\
                                     "type {} to type {}".format(
                        subname, type(subarg), subtype))
            else:
                raise AssertionError("expected argument '{}' of type {}."\
                                     " got {}".format(
                    subname, subtype, type(subarg)))

    return subarg

def check_value(subname, subtypes, subvalues, subarg):
    """
    Check the ``subarg`` against the ``"values"`` ref.
    """
    if isinstance(subtypes, ChoiceOfOne):
        subtype = type(subarg)
        subtype = type_to_string(subtype)
        subvalues = subvalues[subtype]

    # whitelisting
    if isinstance(subvalues, list):
        check_value_whitelist(subname, subvalues, subarg)

    # number ranging (int and float)
    elif ((isinstance(subvalues, str) or
           isinstance(subvalues, unicode)) and
          subvalues.startswith("range")):
        check_value_range(subname, subvalues, subarg)

    # number gt, lt, gte, lte
    elif ((isinstance(subvalues, str) or
           isinstance(subvalues, unicode)) and
          (subvalues.startswith(">") or
           subvalues.startswith("<") or
           subvalues.startswith("(") or
           subvalues.startswith("!"))):
        check_value_comparison(subname, subvalues, subarg)

def check_value_whitelist(subname, subvalues, subarg):
    """
    Check the ``subarg`` against a ``subvalues``.

    :param str subname: the name of the argument (for error parsing)
    :param list subvalues: a list of allowed values
    :param subarg: the value of the argument
    :raises AssertionError: if ``subarg`` not in ``subvalues``
    """
    try:
        assert subarg in subvalues
    except AssertionError as e:
        raise AssertionError("`{}` is not a permitted value for"\
                             " arg '{}'. expected value from"\
                             " {}".format(subarg, subname, subvalues))

def check_value_range(subname, subvalues, subarg):
    """
    Check the ``subarg`` against a number range.

    :param str subname: the name of the argument (for error parsing)
    :param str subvalues: a string representing the range: ``"range(min, max)"``
        .  Note that the range is inclusive!
    :param subarg: the value of the argument
    :raises AssertionError: if ``subarg`` is not within number range
    """
    # number ranging (int and float)        
    ran = eval(subvalues.replace("range", ""))
    try:
        assert ran[0] <= subarg and subarg <= ran[-1]
    except AssertionError as e:
        raise AssertionError("`{}` is not in range({}, {}) for"\
                             " arg '{}'".format(subarg,
                                                ran[0],
                                                ran[-1],
                                                subname))

def check_value_comparison(subname, subvalues, subarg):
    """
    Check the ``subarg`` against the comparison string in ``subvalues``.

    :param str subname: the name of the argument (for error parsing)
    :param str subvalues: the string containing the comparisons to assert.
        eg. ``"((<10||>10)&&!=5)||(<=0&&!=-3)"``
    :param subarg: the value of the argument
    :raises AssertionError: if the assertion of the ``subvalues`` string fails.
    """
    subvalues = subvalues.replace("||", " or ")
    subvalues = subvalues.replace("&&", " and ")
    subvalues = subvalues.replace("<", "{0}<")
    subvalues = subvalues.replace(">", "{0}>")
    subvalues = subvalues.replace("!=", "{0}!=")
    subvalues = subvalues.format(subarg)

    try:
        assert eval(subvalues)
    except AssertionError as e:
        raise AssertionError("`{}` is not `{}` for arg {}".format(
            subarg, subvalues, subname))




from __future__ import absolute_import

from canonical_args import arg_spec, check_args, register_spec
import json


class Obj(object):
	pass

@arg_spec(
	{
		"args": [
			{
				"name": "arg1",
				"type": str,
				"values": [
					"SAMME",
					"SAMME.R"
				]
			},
			{
				"name": "arg2",
				"type": "testfunction.Obj",
				"values": None
			}
		],
	}
)
def test(arg1, arg2):
	print arg1, arg2


@arg_spec(
	{
		"args": [
			{
				"name": "arg1",
				"type": int,
				"values": ">0"
			},
			{
				"name": "arg2",
				"type": dict({
					"arg2.1": {
						"type": int,
						"values": None
					},
					"arg2.2": {
						"type": float,
						"values": ">=0"
					}
				}),
				"values": None
			}
		],
		"kwargs": {
			"kwarg1": {
				"type": dict({
					"thing1": {
						"type": int,
						"values": None
					},
					"thing2": {
						"type": int,
						"values": None
					}
				}),
				"values": None
			}
		}
	}
)
def test2(arg1, arg2, kwarg1=None):
	print arg1, arg2, kwarg1

def test3(*args, **kwargs):
	with open("example.json") as f:
		spec = json.load(f)
	check_args(spec, args, kwargs)

	print args, kwargs
